#!/usr/bin/env node
'use strict'

/*  CONTENTS |

  ntm.0.0 | Keywords:
          syncs
          asyncs
          refers
          modules

  ntm.1.0 | Read File & Translate:
          generate modules[]
          extract task steps
          compile steps to command

  ntm.2.0 | Process:
          INTEGRATING COMMAND PROCESSING HAS NOT BEGUN.
          IN THIS STEP, THE GENERATED COMMAND FROM
          FS.readFile() WILL BE SENT DIRECTLY TO THE
          SYSTEM FOR EXECUTION.


  ntm... | Progressive Notes:
          notes
          todos
          ideas

*/

//===================================================================
// ntm.0.0 | Keywords ===============================================


/* concurrentenators™ | Indicate that the following
task is to be executed concurrently. */
let syncs = ['++', 'WHILE', 'while', 'AND', 'and', '&'];

/* chainers | Indicate that the following task is
    to be executed only after the preceeding task is done. */
let asyncs = ['>>', 'THEN', 'then', '&&'];

/* backreferences | Allow the user to easily reference
the most previous file or directory for further tasking. */
let refers = ['it', 'them', 'origin', 'origins'];

/* modules | Dynamically generated array of user-indicated
modules that are required for the execution of this task. */
let modules = [];


//===================================================================
// ntm.1.0 | Read file & translate ==================================

const FS = require('fs'); // FILESYSTEM

let task = process.argv[2];

FS.readFile('tasks.ntm', 'utf8', (error, data) => {
  error ? console.log('INVALID FILE') : true;

  //  x| Create array of requirements, store in modules[].
  //  x| Locate the task definition. Extract each 'bit' into an array.
  //  x| Translate array, bit by bit, into command to be processed.
  
  
  //.................................................................
  //  x| Create array of requirements, store in modules[] ...........
  modules = data.slice(0, data.indexOf(']'))
    .replace(/\[|\]|,/gmi, ' ')
    .split(/\s/g)
    .filter(Boolean);
  // console.log(modules);



  //..................................................................
  //  x| Locate the task definition. Extract each 'bit' into an array.
  let taskBits = (() => {
    let bits = data.toLowerCase()
      .slice(data.indexOf(`${task}:`) + `${task}:`.length
      , data.indexOf(`:${task}`) - 1)
      .split(/\s/g)
      .filter(Boolean);
    return bits;
  })(task, data); // close getTaskBits()
  // console.log(taskBits);
  // taskBits.forEach((bit) => console.log(bit));



  //.................................................................
  //  x| Translate array, bit by bit, into command to be processed ..
  let command = 'npm run';
  let paths = [];
  let origin = '';

  taskBits.forEach(bit => {
    // If bit is indicated to be a path...
    if (bit[0] == '[') {
      let thisPath = bit.slice(1, -1);
      if (origin.length <= 1) origin = thisPath;
      paths.push(thisPath);
      command += ` ${thisPath}`;
      return;


    // Or if bit is a declared dependency...
    } else if (modules.indexOf(bit) >= 0) {
      command += ` ${bit}`;
      // console.log(`${bit} is a required module.`);
      return;


      // Or if bit is an async keyword...
    } else if (asyncs.indexOf(bit) >= 0) {
      command += ` &&`;
      // console.log(`${bit} (async) converted to &&`);
      return;


      // Or if bit is a sync keyword...
    } else if (syncs.indexOf(bit) >= 0) {
      command += ` &`;
      // console.log(`${bit} (sync) converted to &`);
      return;


      // Or if bit is a reference keyword...
    } else if (refers.indexOf(bit) >= 0) {
      if (bit == 'it' || bit == 'them') {
        // console.log(`${bit} is a refernece to ${paths[paths.length-1]}`);
        command += ` ${paths[paths.length-1]}`;
        return;
      } else {
        // console.log(`${bit} is a refernece to ${origin}`);
        command += ` ${origin}`
        return;
      } // refers[] if/else


    } else {
      // console.log(`'${bit}' is not legal. Ignored.`);
      return;
    }; // f/../else

  }); // forEach bit
  
  console.log(command);
});





/*###################################################
#####################################################
###################################################*/


// let spawn = require('child_process').spawn;


const exec = require('child_process').exec;
const child = exec('npm run babel ./src/babel/*.es6.js ./dist/js/ ',
  (error, stdout, stderr) => {
    console.log(`stdout: ${stdout}`);
    console.log(`stderr: ${stderr}`);
    if (error !== null) {
      console.log(`exec error: ${error}`);
    }
});